/*
 * pian.c
 */

#include "lcd.h"
#include <avr/io.h>
#include <util/delay.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

#define DO				256.87
#define RE				293.66
#define MI				329.63
#define FA				349.23
#define SOL				392
#define LA				440
#define SI				493.88
#define DO2				523.25

#define OCTAVA          267
float notes[]={ DO, RE, MI, FA, SOL, LA, SI, DO2};
volatile unsigned long currentMillis = 0;
volatile int stPrint = 0;

void printOctave(int currentOctave){

    LCD_printAt(LCD_INSTR_nextLine, "             ");
    switch(currentOctave){
        case 0: 
            LCD_printAt(LCD_INSTR_nextLine, "Octava 1"); break;
        case OCTAVA: 
            LCD_printAt(LCD_INSTR_nextLine, "Octava 2"); break;
        case 2*OCTAVA:
            LCD_printAt(LCD_INSTR_nextLine, "Octava 3"); break;
        case 3*OCTAVA:
            LCD_printAt(LCD_INSTR_nextLine, "Octava 4"); break;
    }
}

ISR(TIMER0_COMPA_vect)
{
	currentMillis++;
}

// numara microsecunde
void initTimer0()
{
	OCR0A = 16;
	TCCR0A = (1 << WGM01);
	TCCR0A = (1 << CS00);
	TIMSK0 = (1 << OCIE0A);
}


void initTimer1()
{
	/*buzerul nu scoate niciun sunet*/
	ICR1 = OCR1A = 0;
	
	
	DDRD |= (1 << PORTD5);
	
	TCCR1A = (1 << COM1A1) | (1 << WGM11);
	TCCR1B = (1 << WGM13) | (1 << WGM12) | (1 << CS11) | (1 << CS10);
}

/*seteaza nota data prin frecventa*/
inline void tone(float freq)
{
	/*prescaler - 250 */
	/* 1/freq -> secunde * 1000 => ms */
	ICR1 = 250000 * (1/freq);
	OCR1A = ICR1 / 2;
	
	if(stPrint == 0){
	    printOctave(0);
	    stPrint = 1;
	}
}

/*niciun sunet*/
inline void noTone()
{
	ICR1 = OCR1A = 0;
}

void init() 
{

    /*initializez pini*/
    DDRB &= ~((1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) |

                (1 << PB4) | (1 << PB5) | (1 << PB6) | (1 << PB7));

    PORTB |= (1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) |

                (1 << PB4) | (1 << PB5) | (1 << PB6) | (1 << PB7);
                
    DDRD &= ~((1 << PD0) | (1 << PD1));
    
    PORTD |= (1 << PD0) | (1 << PD1);
		
	/*initializare timere*/
	initTimer0();
	initTimer1();
}

void printNote(int val)
{
    LCD_printAt(0, "Nota: ");
    switch(val){
	case 0: LCD_print("DO1"); break;
	case 1: LCD_print("Re"); break;
	case 2: LCD_print("MI"); break;
	case 3: LCD_print("FA"); break;
	case 4: LCD_print("SOL"); break;
	case 5: LCD_print("LA"); break;
	case 6: LCD_print("SI"); break;
	case 7: LCD_print("DO2"); break;
    }
}

int main()
{
    init();
    LCD_init();
    LCD_print("Hello!");
    LCD_printAt(LCD_INSTR_nextLine, "Fruits Piano!");

    int print = -1;
    int oldPrint = -1;
    int currentOctave = 0;
    int canPress = 1;
    
    for(;;){
    
        if(!(PINB & (1 << PB0))){
            tone(notes[0] + currentOctave);
            print = 0;
        }
        else if(!(PINB & (1 << PB1))){
            tone(notes[1] + currentOctave);
            print = 1;
        }
        else if(!(PINB & (1 << PB2))){
            tone(notes[2] + currentOctave);
            print = 2;
        }
        else if(!(PINB & (1 << PB3))){
            tone(notes[3] + currentOctave);
            print = 3;
        }
        else if(!(PINB & (1 << PB4))){
            tone(notes[4] + currentOctave);
            print = 4;
        }
        else if(!(PINB & (1 << PB5))){
            tone(notes[5] + currentOctave);
            print = 5;
        }
        else if(!(PINB & (1 << PB6))){
            tone(notes[6] + currentOctave);
            print = 6;
        }
        else if(!(PINB & (1 << PB7))){
            tone(notes[7] + currentOctave);
            print = 7;
        } else {
               /*Nu e apasat niciun buton*/
               noTone();
        }
        
        if(print != oldPrint){
            LCD_printAt(0, "              ");
            printNote(print);
            oldPrint = print;
        }
        
        /*Modificare Octava*/
        if(!(PIND & (1 << PD0)) && canPress == 1){
            canPress = 0;       
            if(currentOctave > 0)
                currentOctave-=OCTAVA;
            printOctave(currentOctave);
        } else if(!(PIND & (1 << PD1)) && canPress == 1){
            canPress = 0;
            if(currentOctave < 3 * OCTAVA)
                currentOctave+=OCTAVA;
            printOctave(currentOctave);
        }
        
        if((PIND & (1 << PD0)) && (PIND & (1 << PD1)))
            canPress = 1;
    }
    
    return 0;
}

